"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const http_errors_1 = require("http-errors");
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const validator_1 = __importDefault(require("./validator"));
const utils_1 = require("./utils");
class KeyValueRepository {
    tableName;
    keyName;
    idOptions;
    docClient;
    /**
     * Create a HashKey Repository
     * @param {Object} param - The constructor parameter
     * @param {string} param.tableName - The name of your DynamoDB table
     * @param {string} param.keyName - The name of your Hash/Partition Key property
     * @param {Object} param.documentClient - Injectable DynamoDBDocumentClient (v3) from @aws-sdk/lib-dynamodb
     * @param {Object} [param.idOptions] - The idOptions parameter
     * @param {string} [param.idOptions.prefix=] - The prefix of your id
     */
    constructor(input) {
        (0, validator_1.default)(input);
        const { tableName, keyName, idOptions, documentClient } = input;
        this.tableName = tableName;
        this.keyName = keyName;
        this.idOptions = idOptions;
        this.docClient = documentClient;
    }
    async get(hashKey) {
        const key = { [this.keyName]: hashKey };
        const getParams = {
            TableName: this.tableName,
            Key: key,
        };
        const { Item } = await this.docClient.send(new lib_dynamodb_1.GetCommand(getParams));
        if (!Item) {
            const message = `No ${this.keyName} found for ${hashKey}`;
            throw (0, http_errors_1.NotFound)(message);
        }
        return Item;
    }
    async getMany(input = {}) {
        const { limit = 100, cursor } = input;
        const scanParams = {
            TableName: this.tableName,
            Limit: limit,
        };
        if (cursor) {
            scanParams.ExclusiveStartKey = (0, utils_1.parseCursor)(cursor);
        }
        let result;
        try {
            result = await this.docClient.send(new lib_dynamodb_1.ScanCommand(scanParams));
        }
        catch (err) {
            if (err.code === 'ValidationException') {
                throw new http_errors_1.BadRequest('cursor is not valid');
            }
            throw err;
        }
        const returnCursor = result.LastEvaluatedKey
            ? (0, utils_1.createCursor)(result.LastEvaluatedKey)
            : undefined;
        return {
            items: result.Items || [],
            cursor: returnCursor,
        };
    }
    async remove(hashKey) {
        const key = { [this.keyName]: hashKey };
        const deleteParams = {
            TableName: this.tableName,
            Key: key,
        };
        await this.docClient.send(new lib_dynamodb_1.DeleteCommand(deleteParams));
    }
    async create(item) {
        const itemToSave = { ...item };
        itemToSave[this.keyName] = await (0, utils_1.createId)(this.idOptions);
        const isoString = new Date().toISOString();
        itemToSave.createdAt = isoString;
        itemToSave.updatedAt = isoString;
        itemToSave.revision = 1;
        const putParams = {
            TableName: this.tableName,
            Item: itemToSave,
        };
        await this.docClient.send(new lib_dynamodb_1.PutCommand(putParams));
        return itemToSave;
    }
    async update(item) {
        validateHashKeyPropertyExists({ item, keyName: this.keyName });
        const itemToSave = setRepositoryModifiedProperties(item);
        const { revision: previousRevision } = item;
        const putParams = {
            TableName: this.tableName,
            Item: itemToSave,
            ConditionExpression: 'attribute_exists(#key) AND revision = :prevRev',
            ExpressionAttributeNames: {
                '#key': this.keyName,
            },
            ExpressionAttributeValues: {
                ':prevRev': previousRevision,
            },
            ReturnValuesOnConditionCheckFailure: 'ALL_OLD',
        };
        try {
            await this.docClient.send(new lib_dynamodb_1.PutCommand(putParams));
        }
        catch (err) {
            if (err.name === 'ConditionalCheckFailedException') {
                const { Item } = err;
                if (isNotFoundConflict(Item)) {
                    throw (0, http_errors_1.NotFound)();
                }
                if (isRevisionConflict({
                    expectedRevision: previousRevision,
                    actualRevision: Item?.revision?.N,
                })) {
                    throw (0, http_errors_1.Conflict)(`Conflict: Item in DB has revision [${Item?.revision?.N}]. You are using revision [${previousRevision}]`);
                }
            }
            throw err;
        }
        return itemToSave;
    }
}
const isNotFoundConflict = (itemFromError) => !itemFromError;
const isRevisionConflict = (input) => {
    const { expectedRevision, actualRevision } = input;
    return expectedRevision !== actualRevision;
};
const validateHashKeyPropertyExists = (input) => {
    const { item, keyName } = input;
    if (!item[keyName]) {
        throw new http_errors_1.BadRequest();
    }
};
const setRepositoryModifiedProperties = (item) => {
    const returnItem = { ...item };
    returnItem.updatedAt = new Date().toISOString();
    returnItem.revision = item.revision + 1;
    return returnItem;
};
exports.default = KeyValueRepository;
//# sourceMappingURL=keyValueRepository.js.map